package com.contextspace.objectdb_example.dao.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.contextspace.objectdb_example.dao.exception.DaoException;
import com.contextspace.objectdb_example.model.IDHolder;
import com.googlecode.genericdao.dao.jpa.GenericDAOImpl;
import com.googlecode.genericdao.search.jpa.JPASearchProcessor;

/**
 * Extension of GenericDAOImpl that is configured for Autowiring with Spring or
 * J2EE. This DAO implementation can use any EntityManager implementation.
 * 
 * This DAO also implements all the methods in {@link IBaseDAO} which will be used by any concrete implementation.
 */
@Repository
public class BaseDAOImpl<T, ID extends Serializable> extends GenericDAOImpl<T, ID>{
	static Logger log = LoggerFactory.getLogger(BaseDAOImpl.class.getName());
	
	@Override
	@PersistenceContext(unitName="contextSpacePU")
	public void setEntityManager(EntityManager entityManager) {
		super.setEntityManager(entityManager);
	}
	
	public EntityManager getEntityManager() {
	    return em();
	}

	@Autowired
	@Override
	public void setSearchProcessor(JPASearchProcessor searchProcessor) {
		super.setSearchProcessor(searchProcessor);
	}
	
	public void detach(T entity) {
		em().detach(entity);
	}
	
	/* (non-Javadoc)
     * @see com.contextspace.dao.IBaseDAO#create(T)
     */
	public T create(T entityToCreate) {
		return super.save(entityToCreate);
	}
	
	/* (non-Javadoc)
     * @see com.contextspace.dao.IBaseDAO#create(T...)
     */
	public List<T> create(T... entities) {
		List<T> createdEntities = new ArrayList<T>();
		for(T entity:entities) {
			createdEntities.add(super.save(entity));
		}
		return createdEntities;
	}

	public T lookupById(long id) throws DaoException {
        TypedQuery<T> query = em().createQuery(
                "SELECT DISTINCT t FROM " + persistentClass.getSimpleName() + " t WHERE id=:id", persistentClass); 
        query.setParameter("id", id);
       
        List<T> results = query.getResultList();
        if(results.size() > 1) {
            throw new DaoException("Somehow managed to find more than one object for database id " + id + " - this should be impossible");
        }
        if(log.isDebugEnabled()) {
            log.debug("Results: " + results.size() + " - "+ results);
        }
        
        return results.get(0);
    }
	
	/* (non-Javadoc)
     * @see com.contextspace.dao.IBaseDAO#delete(T)
     */
	public boolean delete(T entityToDelete) {
		return deleteByID(((IDHolder)entityToDelete).getId());
	}

	/* (non-Javadoc)
     * @see com.contextspace.dao.IBaseDAO#deleteById(long)
     */
	public boolean deleteByID(long id) {
		try {
			T attachedEntity = em().getReference(persistentClass, id);
			em().remove(attachedEntity);
			return true;
		} catch (Exception e) {
			log.warn("Could not delete entity. Error message is: " + e.getMessage());
		}
		return false;
	}
	
	/* (non-Javadoc)
     * @see com.contextspace.dao.IBaseDAO#delete(T...)
     */
	public boolean delete(T... entitiesToDelete) {
		try {
			for(T entityToDelete:entitiesToDelete) {
				T attachedEntity = em().getReference(persistentClass, ((IDHolder)entityToDelete).getId());
				em().remove(attachedEntity);
			}
			return true;
		} catch (Exception e) {
			log.warn("Could not delete entities. Error message is: " + e.getMessage());
		}
		return false;
	}
	
	/* (non-Javadoc)
     * @see com.contextspace.dao.IBaseDAO#delete(T...)
     */
	public boolean delete(List<T> entitiesToDelete) {
		try {
			for(T entityToDelete:entitiesToDelete) {
				T attachedEntity = em().getReference(persistentClass, ((IDHolder)entityToDelete).getId());
				em().remove(attachedEntity);
			}
			return true;
		} catch (Exception e) {
			log.warn("Could not delete entities. Error message is: " + e.getMessage());
		}
		return false;
	}

	/* (non-Javadoc)
     * @see com.contextspace.dao.IBaseDAO#update(T)
     */
	public T update(T entityToUpdate) {
		return super.save(entityToUpdate);
	}
	
	/* (non-Javadoc)
     * @see com.contextspace.dao.IBaseDAO#update(T...)
     */
	public List<T> update(T... entitiesToUpdate) {
		List<T> updatedEntities = new ArrayList<T>();
		for(T entity:entitiesToUpdate) {
			updatedEntities.add(super.save(entity));
		}
		return updatedEntities;
	}
	
	/**
	 * Get the count of how many objects are in the database
	 * @return
	 */
	public long count() {
	    CriteriaBuilder qb = em().getCriteriaBuilder();
	    CriteriaQuery<Long> cq = qb.createQuery(Long.class);
	    cq.select(qb.count(cq.from(persistentClass)));
	    return em().createQuery(cq).getSingleResult();
	}
	
	/**
	 * Get a list of all the entities of the specified class.
	 */
	@Override
	public List<T> findAll() {
		log.debug("findAll: Persistence Class is of type " + persistentClass.getName());

		TypedQuery<T> q = em().createQuery("select t from " + persistentClass.getName() + " t", persistentClass);
		List<T> result = q.getResultList();
		
	    log.debug("findAll: Number Of objects for " + persistentClass.getName() + " is " + result.size());
	    
	    return result;
	}   
}	
